#!/usr/bin/env python

"""
Errors reported by the Cora commands
"""

# ---------------------------------------------------------------------------#
class CoraError(Exception):
    FAILURES = {
        'unsupported message': 'command not supported by LoggerNet server',
        'invalid security': 'Server security prevented this command/transaction from executing.',
        'orphaned session': 'The connection to the server was lost while this command was executing.',
        'connection lost': 'The connection to the server was lost while this command was executing.',
        'Expected the device name': 'The name of the device is expected as the first argument.',
        'unknown': 'The server sent a response code that corascript is unable to recognise',
        'session failure': 'The server connection was lost while the transaction was executing.',
        'invalid device name': "The device name specified does not exist in the server's network map.",
        'blocked by server': 'Server security prevented the command from executing.',
        'unsupported': 'The server or the specified device does not support the command/transaction.',
        'blocked by logger': 'The security code setting for the specified device is not valid.',
        'communication disabled': 'Communication with the datalogger is disabled.',
        'communication failed': 'Communication with the datalogger failed.',
        'Expected the account name': 'The name of the account was expected in the first argument.',
        'Expected the account password': 'The password for the account was expected in the second argument.',
        'Expected the access level': 'The access level for the account was expected in the third argument.',
        'Invalid access level specified': 'An invalid access level was speciifed.',
        'unknown failure': 'The server sent a failure code that was not recognised by corascript.',
        'insufficient access to add accounts': 'Server security blocked this command from executing.',
        'connection failed': 'The server connection was lost while this command was executing.',
        'security interface is locked': 'Another client has locked the security interface.',
        'invalid account name': 'An invalid (or already existing) account name was specified.',
        'account is in use ': 'The command referred to an account that is currently being used.',
        'insufficient access to delete accounts': 'The command failed because of server security.',
        'Broker name expected first': 'The name of the data broker was expected as the first argument.',
        'invalid broker specified': 'There is no data broker that has the specified name.',
        'unsupported message type': 'The server does not support this command.',
        'exception': 'The server sent a response code that corascript was unable to recognise.',
        'Expected the column identifier': 'The column identifier was expected as the first argument.',
        'server_security_blocked': 'Server security prevented the command from executing.',
        'invalid_table_name': 'The table name specified does not exist.',
        'invalid_column_name': 'The column name specified does not exist.',
        'invalid_subscript': 'The array subscript specified does not exist.',
        'communication_failed': 'Communication with the datalogger failed.',
        'communication_disabled': 'Communication with the datalogger is disabled.',
        'logger_security_blocked': 'The security code setting for the logger device is not '
                                   'set to an appropriate value.',
        'invalid_table_definitions': "The server's table definitions are not valid.",
        'invalid_device_name': 'The name of the device specified is invalid.',
        'unsupported by the server': 'The transaction is not supported by the server or by the device specified.',
        'Invalid Action code': 'The value of the action option is invalid.',
        'unknown error': 'The server sent a response code that corascript was unable to recognise.',
        'in progress': 'Another transaction is already in progress.',
        'rejected security code': 'The security code setting for the device is wrong.',
        'communication failure': 'Communication with the datalogger failed.',
        'network is locked': 'Another client has the network locked.',
        'network locked': 'Another client has the network locked.',
        'communications disabled': 'Communication with the datalogger is disabled.',
        'Expected the setting identifier': 'Expected the setting identifier as the third argument.',
        'expected the setting value': 'Expected the new value for the setting as the fourth argument.',
        'server session broken': 'The server session was broken while this command was executing.',
        'unsuported transaction': 'One or more required transactions are not supported by the server.',
        'blocked by server security': 'The command could not execute because of server security.',
        'device is online':
            'The command cannot execute because the device (or one of its children) is on-line. Current '
            'versions of the server will force all of the effected devices off-line.',
        'Invalid value name syntax': '',
        'unsupported setting identifier': 'corascript does not recognise the specified setting identifier',
        'invalid setting format': 'corascript was unable to parse the setting.',
        'invalid device name specified': 'A device name was specified that is not present in the servers network map.',
        'session failed': 'The session with the specified device faile while the command was pending. This could '
                          'happen because the device was deleted or because the server was shut down while the command '
                          'was pending.',
        'server security blocked': 'Server security prevented the command from being carried out.',
        'invalid setting value': 'An invalid value was specified for this setting.',
        'setting is read-only': 'The specified setting cannot be set.',
        'Expected the file name': 'The name of the file to send was expected as the second argument.',
        'Invalid run-now option specified': 'The value associated with the run-now option was invalid.',
        'Invalid run-on-power-up option specified': 'The value associated with the run-on-power-up option was invalid.',
        'invalid file name': 'The file name specified is invalid.',
        'logger resource error': 'The datalogger does not have the resources to store the file.',
        'logger compile error': 'The datalogger could not compile the program.',
        'logger permission denied': 'The security code setting for the specified device is not set to a valid value.',
        'server connection failed': 'The connection to the server failed while this command was executing.',
        'server permission denied': 'Server security prevented this command from executing.',
        'Device name expected first': 'The name of the datalogger type device was expected as the first argument.',
        'Unable to open the file': 'The specified file could not be opened for reading.',
        'Expected the file path in argument 2': 'The name of the file to send was expected as the second argument.',
        'logger locked': 'The server is engaged in a critical transaction with the datalogger and cannot start this '
                         'command at this time.',
        'server resource error': 'The server does not have the resources to store the file.',
        'communication is disabled': 'Communication with the specified device is disabled.',
        'unknown error code': 'The server sent a response code that corascript was unable to recognise.',
        'compile failure': 'The datalogger could not compile the program.',
        'datalogger communication failure': 'Communication failed with the datalogger',
        'security failure': 'The security code setting for the datalogger device is set to an invalid value.',
        'datalogger storage buffer full': 'The datalogger does not have enough space to store the entire program.',
        'datalogger communication is disabled': 'Communication with the datalogger is disabled.',
        'table definitions read failure': 'The server was unable to read new table definitions from the datalogger '
                                          'following the program file send.'
    }

    def __init__(self, value):
        self.value = CoraError.FAILURES[value]

    def __str__(self):
        return self.value



#!/usr/bin/env python

"""
Utilities used in the module
"""

import re
from logging import getLogger
import os

from corautil.errors import CoraError


def format_cora_backup_scripts(filename):
    """
    remove tabs, extra spaces and extraneous new-lines in
    create-backup-script's created by cora

    :param filename: cora backup script to format
    :type filename: str
    :return
    """

    buffer_str = ""
    parsed_file = []

    with open(filename, 'r') as fid:
        for line in fid:
            # remove the line endings
            line = line.rstrip()

            # remove whitespace
            line = re.sub(r'\s+', ' ', line)
            # strip spaces that at the end of the line
            line = line.rstrip(' ')
            # if the cora command is already on a single line just append it to the list
            if line.endswith(';') and buffer_str == "":
                parsed_file.append(line)

            # finish building the cora command and append it to the list when the semi-colon is found
            elif line.endswith(';') and buffer_str != "":
                buffer_str += ' ' + line
                parsed_file.append(buffer_str)
                buffer_str = ""

            # start building the cora command if the line does not contain a semi-colon
            else:
                # append a space to the line if this is not the first line
                if buffer_str != "":
                    buffer_str += ' '

                buffer_str += line

    with open(filename, 'w') as fid:
        fid.write('\n'.join(parsed_file))

    return parsed_file


def remove_quotes(str_list):
    if type(str_list) is list:
        for index, line in enumerate(str_list):
            line = re.sub(r'"', '', line)
            str_list[index] = line

    return str_list


def extract_data(str, command):
    logger = getLogger('corautil.extract_data')

    resp_regex = re.compile(
        r"\{\s(?P<response>.+|\B)\s\}",
        # r"\*" + command + r"(?:\s|,active\s|,ignore\s)\{\s(?P<response>.+|\B)\s\}\s",
        flags=re.DOTALL
    )
    response = resp_regex.search(str)
    if response is not None:
        logger.debug(response.group('response'))

        return response.group('response').strip().split('\n')



#!/usr/bin/env python

"""
Device Settings that can be changed via cora
"""


class DeviceSettings(object):
    Settings = {
        'allowedPakbusNeighbours': 93,
        'airlinkModemId': 87,
        'baudRate': 11,
        'bmp1LowLevelDelay': 53,
        'bmp1MutexName': 47,
        'bmp1StatId': 16,
        'bmp5CallbackEnabled': 80,
        'cacheIpAddress': 88,
        'callbackEnabled': 36,
        'callbackIdentifier': 37,
        'clkChkSched': 1,
        'collectPortsAndFlags': 84,
        'collectSched': 5,
        'collectViaAdvise': 18,
        'commEnabled': 22,
        'comPortId': 15,
        'createCacheTablesOnlyInMemory': 92,
        'currentProgramName': 89,
        'dataBrokerId': 34,
        'dataCollectionEnabled': 28,
        'defaultCsixmlFormatOptions': 100,
        'defaultCustomCsvFormatOptions': 94,
        'defaultDataFileOutputFormat': 61,
        'defaultDataFileOutputName': 59,
        'defaultDataFileOutputOption': 58,
        'defaultDataFileTimeStampResolution': 60,
        'defaultDataFileToaHeaderFormat': 62,
        'defaultNohFormatOptions': 99,
        'defaultToa5FormatOptions': 97,
        'defaultTob1FormatOptions': 98,
        'defaultCacheData': 57,
        'defaultScheduleEnabled': 56,
        'defaultTableFileFormat': 81,
        'delayCommsAfterOpen': 85,
        'delayHangup': 83,
        'deleteFilesAfterSynch': 118,
        'dialStrList': 24,
        'doHoleCollect': 10,
        'dunEntryName': 108,
        'extraRespTime': 4,
        'fileSynchControl': 96,
        'fileSynchControlEx': 114,
        'fileSynchMode': 111,
        'fileSynchScheduleBase': 112,
        'fileSynchScheduleInterval': 113,
        'ftpAuthorisation': 107,
        'genericDialScript': 48,
        'genericEndScript': 49,
        'genericHalfDuplex': 50,
        'genericRaiseDtr': 51,
        'genericRtsCtsUse': 52,
        'holeAdditionEnabled': 42,
        'inputLocationLabels': 38,
        'lgrProgInf': 13,
        'lowLevelPoll': 14,
        'lowLevelPollEnabled': 101,
        'maxBaudRate': 70,
        'maxCacheTableSize': 91,
        'maxInlocsPerRequest': 35,
        'maxPktSize': 3,
        'maxTimeOnLine': 2,
        'pakbusBeaconInterval': 71,
        'pakbusComputerId': 79,
        'pakbusEncryptionKey': 130,
        'pakbusIsDialedLink': 72,
        'pakbusLeafNode': 95,
        'pakbusNodeIdentifier': 55,
        'pakbusTcpMaintainedNodes': 110,
        'pakbusTcpOutAddresses': 115,
        'pakbusVerifyInterval': 109,
        'phoneModemType': 25,
        'pakbusRouterName': 86,
        'pollForStatistics': 129,
        'pooledSerialPorts': 123,
        'pooledTerminalServers': 124,
        'preventTcpOpen': 102,
        'rescheduleOnData': 117,
        'rf400AttentionChar': 75,
        'rf400NetworkId': 73,
        'rf400RadioId': 74,
        'rf95CustomDialString': 77,
        'rf95DialRetries': 76,
        'rfTdPollInterval': 121,
        'rfUseF': 39,
        'rfUseU': 40,
        'rfUseW': 41,
        'rootDelayBeforeReopen': 69,
        'rwisFtpPort': 119,
        'rwisFtpUsePassive': 120,
        'secondaryCollectScheduleEnabled': 67,
        'securityCode': 6,
        'serialUseSimplifiedIo': 122,
        'settingsOverriden': 27,
        'snmpAgentAddress': 104,
        'snmpReadCommunity': 105,
        'snmpReadWriteCommunity': 106,
        'socketPostCloseScript': 128,
        'socketPreOpenScript': 127,
        'stayOnCollectSchedule': 68,
        'switchId': 12,
        'tableDefsPolicy': 78,
        'tableFileStationNameSelector': 126,
        'tablesToExclude': 19,
        'tableSizeFactor': 21,
        'tapiAreaCode': 65,
        'tapiCountryCode': 64,
        'tapiDialString': 66,
        'tcpCallbackPort': 82,
        'tcpCallbackVerifyTime': 103,
        'tcpPassword': 116,
        'timeZoneDiff': 20,
        'userDescription': 90,
        'useTapiDialingProperties': 63
    }

    def __init__(self, setting):
        self.value = DeviceSettings.Settings[setting]

    def __int__(self):
        return self.value



import pytest
from logging import getLogger, DEBUG, INFO

from corautil.device_settings import DeviceSettings


@pytest.mark.parametrize(('setting', 'setting_id'),
                         [
                             ('callbackEnabled', 36),
                             ('currentProgramName', 89),
                         ]
                         )
def test_device_settings(setting, setting_id):
    logger = getLogger('test_device_settings')
    logger.setLevel(DEBUG)

    set_id = DeviceSettings(setting).value

    logger.debug('{}: {}'.format(setting, set_id))

    assert set_id == setting_id



import pytest
import logging
from logging import getLogger, DEBUG, INFO
import filecmp
import pathlib
from os import path

from corautil.corautil import CoraUtil

base_dir = path.dirname(__file__)


@pytest.mark.parametrize(('station_name', 'filename'),
                         [
                             ('Framingham_Interchange_13_South', 's3621_framingham-13-south_das-1.CR8'),
                             ('Framingham_Interchange_13_South', 's3621_framingham-13-south.cfg'),
                             # ('Framingham_Interchange_13_South', '.draker_id')
                         ]
                         )
def test_get_file(station_name, filename):
    logger = getLogger('test_get_file')
    logger.setLevel(DEBUG)

    console = logging.StreamHandler()
    console.setLevel(logging.DEBUG)
    console.setFormatter(logging.Formatter('%(name)-12s: %(levelname)-8s %(message)s'))
    logger.addHandler(console)

    loggernet = CoraUtil('devops.inaccess.com', '', '')
    loggernet.set_logger_level(DEBUG)

    loggernet.get_file(station_name, filename)

    assert filecmp.cmp(filename, filename + '.orig')



#!/usr/bin/env python

"""
LoggerNet Settings that can be changed via cora
"""


class LgrNetSettings(object):
    Settings = {
        'applicationDir': 25,
        'allowRemoteTasksAdmin': 28,
        'autoBackupEnabled': 16,
        'autoBackupBase': 17,
        'autoBackupInterval': 18,
        'autoBackupIncludeCache': 19,
        'autoBackupExtraPaths': 20,
        'autoBackupPath': 21,
        'autoBackupBaleCount': 22,
        'bmp1ComputerId': 6,
        'checkPassWd': 8,
        'commEnabled': 7,
        'commsSettings': 3,
        'cqrSettings': 15,
        'defaultClockSchedule': 29,
        'defaultCollectPortsAndFlags': 45,
        'defaultCollectSchedule': 30,
        'defaultCollectViaAdvise': 35,
        'defaultCsixmlFormatOptions': 62,
        'defaultCustomCsvFormatOptions': 58,
        'defaultDataFileOutputName': 52,
        'defaultDataFileOutputOption': 51,
        'defaultDeleteFilesAfterSynch': 44,
        'defaultDoHoleCollect': 33,
        'defaultFileSynchControlEx': 43,
        'defaultFileSynchMode': 40,
        'defaultFileSynchScheduleBase': 41,
        'defaultFileSynchScheduleInterval': 42,
        'defaultFsArraysToCollectOnFirstPoll': 49,
        'defaultFsCollectAllOnFirstPoll': 48,
        'defaultFsCollectMode': 47,
        'defaultFsMaxArraysToPoll': 50,
        'defaultFsOutputFormat': 46,
        'defaultHoleAdditionEnabled': 34,
        'defaultMaxCacheTableSize': 38,
        'defaultNohFormatOptions': 61,
        'defaultPollForStatistics': 65,
        'defaultRescheduleOnData': 36,
        'defaultSecondaryCollectScheduleEnabled': 31,
        'defaultStayOnCollectSchedule': 32,
        'defaultTableCollectAllOnFirstPoll': 54,
        'defaultTableCollectMode': 53,
        'defaultTableDefsPolicy': 37,
        'defaultTableFileFormat': 57,
        'defaultTableFileStationNameSelector': 63,
        'defaultTableMaxIntervalToPoll': 69,
        'defaultTableMaxRecordsToPoll': 56,
        'defaultTableRecordsToCollectOnFirstPoll': 55,
        'defaultTableSizeFactor': 39,
        'defaultToa5FormatOptions': 59,
        'defaultTob1FormatOptions': 60,
        'dirSeparator': 26,
        'ipManagerUdpPort': 13,
        'ipManagerKey': 14,
        'lowSettings': 5,
        'maxDataFileSize': 64,
        'minConfigRewriteInterval': 23,
        'pakbusComputerId': 11,
        'scheduledOn': 1,
        'stateSettings': 4,
        'systemClk': 9,
        'tranSettings': 2,
        'useGlobalPakbusRouter': 12,
        'userNotes': 27,
        'workingDir': 24
    }

    def __init__(self, setting):
        self.value = LgrNetSettings.Settings[setting]

    def __int__(self):
        return self.value



#!/usr/bin/env python

"""
Run Cora commands
"""

# ---------------------------------------------------------------------------#
from argparse import ArgumentParser
from subprocess import check_output, Popen, PIPE
from logging.config import fileConfig

import os
from os import path
import sys
import signal
from logging import getLogger, INFO, DEBUG
import time
import re
from datetime import datetime
import time

from corautil.errors import CoraError
from corautil.utils import format_cora_backup_scripts, remove_quotes, extract_data
from corautil.device_settings import DeviceSettings

__version__ = '1.0.2'

logger = getLogger('corautil')

# ---------------------------------------------------------------------------#
# noinspection PyUnusedLocal,PyUnusedLocal,PyShadowingNames
def signal_handler(signal, frame):
    print ('You pressed Ctrl+C')
    logger.info('Script Stopped on: %s' % time.asctime(
        time.localtime(time.time())))
    sys.exit(0)


# ---------------------------------------------------------------------------#
class CoraUtil:
    """
    Utility to execute cora commands for use in Python scripts
    """
    ACCESS_LEVEL = {
        'read-only': '1000',
        'Read-Only': '1000',
        'operator': '2000',
        'Operator': '2000',
        'manager': '3000',
        'Station Manager': '3000',
        'administrator': '4000',
        'Network Administrator': '4000',
        'root': '5000',
        'Full Administrator': '5000'
    }

    def __init__(self, server_ip, username, password, server_port=6789):
        """
        :param server_ip: IP Address or DNS Name of the LoggerNet server to connect to
        :type server_ip: str
        :param username: username for the account being used to access the LoggerNet server
        :type username: str
        :param password: the password for the account being used to access the LoggerNet server
        :type password: str
        :param server_port:
        :type server_port: int
        :return:
        """

        self.server_ip = server_ip
        self.username = username
        self.password = password
        self.server_port = server_port
        self.logger = getLogger('corautil')
        self.logger.setLevel(INFO)
        self.version = __version__

    def version(self):
        return self.version

    def set_logger_level(self, logger_level):
        self.logger.setLevel(logger_level)
        self.logger.info('Changed logging level to {}'.format(logger_level))

    def execute_cora(self, command_str):
        """
        execute a list of cora commands

        :param command_str: the list of cora commands
        :type command_str: str
        """

        if os.name == 'nt':
            cora = ["C:\Program Files (x86)\Campbellsci\LoggerNet\cora_cmd.exe"]
        elif os.name == 'posix':
            cora = ['/opt/CampbellSci/Loggernet/cora_cmd']
        else:
            cora = []

        # spawn an instance of cora
        cora_proc = Popen(cora, stdin=PIPE, stdout=PIPE, stderr=PIPE, universal_newlines=True)

        #generate the connect command and lock the network
        cora_cmd = 'connect '
        cora_cmd += self.server_ip + ' '
        cora_cmd += '--name={' + self.username + '} '
        cora_cmd += '--password={' + self.password + '} '
        cora_cmd += '--server-port=' + str(self.server_port) + '; '
        cora_cmd += 'lock-network;'
        cora_cmd += command_str
        cora_cmd += 'unlock-network; exit;'

        # run the cora command and terminate the instance
        self.logger.debug('sending cora command: {}'.format(command_str))
        output, output_err = cora_proc.communicate(
            input=cora_cmd
        )

        if cora_proc.returncode != 0:
            self.logger.debug('cora returncode is {}'.format(cora_proc.returncode))
            self.logger.debug('cora error is {}'.format(output_err))
            exit(cora_proc.returncode)

        # terminate the cora subprocess
        cora_proc.kill()

        # create regex to find if cmd error
        command = command_str.partition(' ')
        err_regex = re.compile("-" + command[0].strip(';') + ",(?P<error_message>.+)")

        output = re.sub(r'\r\n', '\n', output)
        self.logger.debug('output of cora is: {}'.format(output))

        if err_regex.search(output) is not None:
            raise CoraError(err_regex.search(output).group('error_message').strip())
        else:
            return output

    def list_stations(self):
        """
        issues the cora command to list the station on the LoggerNet server and return as a list

        :return:
        """
        cmd = 'list-stations'
        station_list = []
        station_name = re.compile(r'\{\{(?P<station_name>.*)\}\s+(?P<broker_id>\d+)\}')

        self.logger.debug('getting station list via cora')

        cora_output = self.execute_cora(cmd + ';')

        try:
            for line in extract_data(cora_output, cmd):
                sn_match = station_name.match(line.strip())
                if sn_match is not None:
                    if sn_match.group('station_name') != '__Statistics__':
                        station_list.append(sn_match.group('station_name'))

            return station_list

        except CoraError:
            raise

    def list_files(self, station_name):
        """
        This command is used to list the files that can be found in a datalogger's file system. It is supported only
        for those datalogger types that support file systems (CR9000, CR5000, CR1000, CR10X-PB, CR510-PB,
        CR23X-PB, and, in a limited sense, the CR2xx).

        :param station_name:
        :return:
        """
        re_file = re.compile(
            r'\{CPU:(?P<filename>.+)\}\srn=(?P<rn>false|true)\spu=(?P<pu>false|true)' +
            r'\sro=(?P<ro>false|true)\ssize=(?P<size>\d+)\slast-changed=\{(?P<last_changed>.+)\}'
        )

        cmd = 'list-files'
        file_list = []

        try:
            cora_output = self.execute_cora(cmd + ' ' + station_name + ';')

            logger.debug('cora_output list is: {}'.format(cora_output))

            for line in extract_data(cora_output, cmd):
                # clear dictionary
                file_info = {}

                # remove newline characters
                line = line.strip()

                logger.debug('cora_output line is: {}'.format(line))

                if 'CPU' in line:

                    parameter = re_file.search(line)
                    # logger.debug('cora_output regex is: {}'.format(parameter))

                    if parameter:
                        # logger.debug('file parameters are: {}'.format(parameter.groups()))

                        file_info['filename'] = parameter.group('filename')
                        file_info['filename'] = file_info['filename'].strip('{CPU:')
                        file_info['filename'] = file_info['filename'].strip('}')

                        file_info['rn'] = parameter.group('rn')
                        file_info['pu'] = parameter.group('pu')
                        file_info['ro'] = parameter.group('ro')
                        file_info['size'] = parameter.group('size')
                        file_info['last-changed'] = parameter.group('last_changed')

                        self.logger.debug('filename is: {}'.format(file_info['filename']))
                        self.logger.debug('file_info is: {}'.format(file_info))

                        if file_info['filename'] != '':
                            file_list.append(file_info)

                        self.logger.debug('the complete list is: {}'.format(file_list))
            return file_list

        except CoraError:
            raise

    def send_file(self, station_name, filepath, run_now=False, run_on_power_up=False):
        """
        This command sends a file from the local computer's file system to a station's file system. The file can
        optionally be marked to run as the current program or on power up. This command is supported only
        for datalogger types that support file systems including the CR9000, CR5000, CR1000, CR800 Series,
        CR3000, and, in a more limited sense, the CR10X-PB, CR510-PB, and CR23X-PB.

        :param station_name:
        :param filename:
        :param run_now:
        :param run_on_power_up:
        :return:
        """

        try:
            if path.exists(filepath):
                filepath = '{' + path.abspath(filepath) + '}'
                send_options = '{' + '--run-now={} --run-on-power-up={}'.format(run_now, run_on_power_up) + '}'

                cora_output = self.execute_cora('send-file {} {} {};'.format(station_name, filepath, send_options))

                return True
            else:
                raise OSError(filepath)

        except CoraError:
            raise
            
    def get_file(self, station_name, filename, storage_location='CPU:', save_as='', save_path=''):
        """
        This command transfers a file from the datalogger's file system to your computer's file system.
        
        :param station_name:
        :param storage_location:
        :param save_as:
        :return:
        """
        
        if save_path == '':
            save_path = os.getcwd()

        try:
            if path.exists(save_path):
                if save_as != '':
                    save_options = '{' + '--save-as={}'.format(path.join(path.abspath(save_path), save_as)) + '}'
                    cora_output = self.execute_cora('get-file {} {} {};'.format(station_name, storage_location + filename, save_options))
                else:
                    cora_output = self.execute_cora('get-file {} {};'.format(station_name, storage_location + filename))

                return True
            else:
                raise OSError(save_path)

        except CoraError:
            os.remove(filename)
            raise

    def send_program_file(self, station_name, filepath, sending_os=False):
        try:
            if path.exists(filepath):

                filename, file_ext = path.splitext(filepath)
                filepath = '{' + path.abspath(filepath) + '}'

                if sending_os and file_ext == '.obj':
                    cora_output = self.execute_cora(
                        'send-program-file {} {} prevent-first-stop;'.format(station_name, filepath))
                else:
                    cora_output = self.execute_cora('send-program-file {} {};'.format(station_name, filepath))

                return True
            else:
                raise OSError(filepath)

        except CoraError:
            raise

    def list_devices(self):
        """
        get a list of the devices on the loggernet server
        """

        device_list = []

        reg_device = re.compile(
            r'\{\{(?P<device_name>.+)\}\s(?P<device_id>\d)+\s(?P<device_type_code>.+)\s(?P<device_indent>\d+)\}'
        )

        try:
            cora_output = self.execute_cora('list-devices;')

            for line in extract_data(cora_output, 'list-devices'):
                line = line.strip()

                device = reg_device.search(line)

                if device:
                    # append the device to list of devices
                    device_list.append(
                        {
                            'device_name': device.group('device_name').strip(),
                            'device_id': device.group('device_id').strip(),
                            'device_type_code': device.group('device_type_code').strip(),
                            'device_indent': device.group('device_indent').strip()
                        }
                    )

            return device_list

        except CoraError:
            raise

    def get_network_map(self, export_format='xml'):
        # execute the cora command
        cora_output = self.execute_cora('make-xml-network-map --format=' + export_format + ';')
        self.logger.info('{}'.format(cora_output))

    def list_tables(self, station_name):
        try:
            cora_output = self.execute_cora('list-tables {' + station_name + '};')
            self.logger.debug('cora_output is: {}'.format(cora_output))
            return remove_quotes(extract_data(cora_output, 'list-tables'))

        except CoraError:
            raise

    def clock_check(self, station_name):
        """
         Returns the current time of the station and the difference between the station and the server.
         The time difference is in milliseconds.  A negative value indicates the station is ahead of the server.

        :param station_name:
        :return:
        """
        station_time = {}

        try:
            cora_output = self.execute_cora('clock-check {' + station_name + '};')

            # extract the time returned
            response = extract_data(cora_output, 'clock-check')

            self.logger.debug(response)

            clock_response = re.search(
                '"(?P<station_time>\d{4}-\d{2}-\d{2} \d{2}:\d{2}:\d{2}.\d{3})",(?P<differences_msec>[|-]\d+)',
                response[0]
            )

            if clock_response is not None:
                station_time['Station Time'] = clock_response.group('station_time')
                station_time['Time Difference'] = clock_response.group('differences_msec')

                return station_time

        except CoraError:
            raise

    def data_query(self, station_name, table_name, begin_date, end_date):
        try:
            cmd = 'data-query {%s} {%s} "%s" "%s";' % (station_name, table_name, begin_date, end_date)
            self.logger.debug('cora command: {}'.format(cmd))

            cora_output = self.execute_cora(cmd)
            self.logger.debug('cora_output is: {}'.format(cora_output))
            return extract_data(cora_output, 'data-query')

        except CoraError:
            raise

    def get_value(self, station_name, value, swath=1):
        try:
            cora_output = self.execute_cora('get-value {' + station_name + '.'+ value + '} --swath=' + str(swath) + ';')

            self.logger.debug('cora_output is: {}'.format(cora_output))
            cora_output = remove_quotes(cora_output)
            return extract_data(cora_output, 'get-value')

        except CoraError:
            raise

    def get_program_stats(self, station_name):
        # list of cora command to execute
        cora_output = self.execute_cora('get-program-stats {{{}}};'.format(station_name))
        self.logger.info('{}'.format(cora_output))

    def get_device_setting(self, device_name, setting):
        try:
            setting_id = DeviceSettings(setting).value
            self.logger.debug('Getting {}({}) from {}'.format(setting, setting_id, device_name))
            cora_output = self.execute_cora('get-device-setting {{{}}} {};'.format(device_name, setting_id))

            settings = extract_data(cora_output, 'get-device-setting')

            if len(settings) == 1:
                return settings[0]
            else:
                settings = list(filter(None, settings))
                return settings

        except CoraError:
            raise

    def set_device_setting(self, device_name, setting, formatted_setting):
        try:
            # list of cora command to execute
            # "set-device-setting" device-name setting-id formatted-setting.
            setting_id = DeviceSettings(setting).value
            self.logger.debug('Getting {}({}) from {}'.format(setting, setting_id, device_name))
            self.execute_cora('set-device-setting {{{}}} {} {{{}}};'.format(device_name, setting_id, formatted_setting))

        except CoraError:
            raise

    def change_account(self, account_name, password, security_level, device_addition=''):
        options = [account_name, password, security_level, '{' + device_addition + '};']
        cora_output = self.execute_cora('change-account ' + ' '.join(options))
        self.logger.info('{}'.format(cora_output))

    def add_account(self, account_name, password, security_level, device_addition=''):
        options = [account_name, password, security_level, '{' + device_addition + '};']
        cora_output = self.execute_cora('add-account ' + ' '.join(options))
        self.logger.info('{}'.format(cora_output))

    def delete_account(self, account_name):
        cora_output = self.execute_cora('delete-account ' + account_name + ';')
        self.logger.info('{}'.format(cora_output))

    def list_accounts(self):
        cora_output = self.execute_cora('list-accounts;')
        self.logger.info('{}'.format(cora_output))

    def add_device(self, device_type, device_name, anchor_code, anchor_device_name):
        # cora_output = self.execute_cora(';')
        pass

    def enable_collection(self, station):
        cora_output = self.execute_cora(
            'set-device-setting %s 5 {true {19900101 00:00:30.000} 900000 120000 5 900000};' % station
        )
        return cora_output

    def disable_collection(self, station):
        cora_output = self.execute_cora(
            'set-device-setting %s 5 {false {19900101 00:00:30.000} 900000 120000 5 900000};' % station
        )
        return cora_output

    def get_table_defs(self, station):
        try:
            cora_output = self.execute_cora('get-table-defs %s;' % station)

            if '+get-table-defs' in cora_output:
                return True

        except CoraError:
            raise

    def collect_table(self, station, table):
        cora_output = self.execute_cora('set-collect-area-setting %s %s 2 true;' % (station, table))
        if cora_output not in dict.keys(CoraError.FAILURES):
            return True
        else:
            return False

    def get_table_settings(self, station, table):
        cora_output = self.execute_cora('list-collect-area-settings %s %s;' % (station, table))
        if cora_output not in dict.keys(CoraError.FAILURES):
            str_start = cora_output.index('*list-collect-area-settings')
            str_end = cora_output.index('+list-collect-area-settings')
            description_str = cora_output[str_start:str_end]
            description_str = re.sub('\n', '', description_str)
            self.logger.debug('{}'.format(cora_output))
            self.logger.debug('{}'.format(str_start))
            self.logger.debug('{}'.format(str_end))
            self.logger.debug('{}'.format(description_str))

            return description_str

        else:
            return cora_output

    def create_backup_scripts(self):
        # create backup
        cora_output = self.execute_cora('create-backup-script ' + self.server_ip + '-backup.cora;')
        parsed_backup = format_cora_backup_scripts(self.server_ip + '-backup.cora')
        return parsed_backup

    def read_note(self, station):
        try:
            # use this to read a station note in loggernet
            cora_output = self.get_device_setting(station, 'userDescription')
            self.logger.debug('cora_output is: {}'.format(cora_output))
            return cora_output

        except CoraError:
            raise

    def write_note(self, station, note, update=True):
        try:
            # use this to write a note to a station in loggernet
            if update:
                current_note = self.read_note(station)
                # if list, make it a single string
                if isinstance(current_note, list):
                    current_note = ' '.join(current_note)

                new_note = current_note + ' ' + note
            else:
                new_note = note

            cora_output = self.set_device_setting(station, 'userDescription', new_note)
            self.logger.debug('cora_output is: {}'.format(cora_output))
        except CoraError:
            raise

    def delete_device(self, device):
        cora_output = self.execute_cora('delete-device ' + device + ';')
        self.logger.info('{}'.format(cora_output))

        return cora_output

# ---------------------------------------------------------------------------#
if __name__ == '__main__':
    fileConfig(os.path.join(os.path.dirname(__file__), 'cora_logger.ini'))
    logger = getLogger('corautil')
    logger.setLevel(DEBUG)

    logger.debug(os.getcwd())

    station = {}

    parser = ArgumentParser()
    parser.add_argument(
        '--server_ip', help='ip address of the LoggerNet server', default='7.loggernet.draker.us'
    )
    parser.add_argument(
        '--username', help='username for LoggerNet server', default='Joe'
    )
    parser.add_argument(
        '--password', help='password for LoggerNet server', default='w4LdL4fe'
    )
    args = parser.parse_args()

    # register Ctrl-C signal handler
    signal.signal(signal.SIGINT, signal_handler)

    if args.username == '':
        args.username = input('Please enter your LoggerNet username')

    if args.password == '':
        args.password = input('Please enter your LoggerNet password')

    # tests for
    # for name, server in dict.items(loggernet_servers):
    #     if name not in ['localhost', 'all']:
    #         logger.info('Performing Operations on {}'.format(name))
    #         loggernet = Cora(server, args.username, args.password)
    #         # loggernet.changeAccount('Joe', 'Schmo', Cora.ACCESS_LEVEL['Full Administrator'])
    #         loggernet.deleteAccount('Joe')

    loggernet = CoraUtil(args.server_ip, args.username, args.password)
    loggernet.set_logger_level(DEBUG)
    # logger.info('{}'.format(loggernet.list_stations()))

    # for station_name in station_list:
    #     station[station_name] = {}
    #     station[station_name]['list-files'] = loggernet.listFiles(station_name)

    # test for listFiles
    # station_name = 'LAB-CR1000'
    # station[station_name] = {}
    # station[station_name]['list-files'] = loggernet.listFiles(station_name)

    # logger.info('{}'.format(station))

    # test for listDevices
    # logger.info('{}'.format(loggernet.list_devices()))

    # logger.info('{}'.format(loggernet.list_tables('draker_dealer-dot-com')))
    # logger.info('{}'.format(loggernet.clock_check('draker_dealer-dot-com')))
    # logger.info('{}'.format(loggernet.list_files('draker_dealer-dot-com')))
    # logger.info('{}'.format(loggernet.get_value('draker_dealer-dot-com', 'Public.datalogger_ip')))
    # logger.info('{}'.format(loggernet.get_value('draker_dealer-dot-com', 'DataTableInfo.DataTableName', 5)))
    # records = loggernet.data_query('draker_dealer-dot-com', 'fifteenMin', '20170601 23:45', '20170602 00:00')
    # records = loggernet.data_query('__statistics__', 'draker_dealer-dot-com_hist', '20170602 20:00', '20170603 00:00')

    # for record in records:
    #     logger.info('{}'.format(record))

    # records = loggernet.data_query('__statistics__', 'draker_dealer-dot-com_std', '20170602', '20170603')
    # records = loggernet.data_query('__statistics__', 'sunwize_aspa-tafuna-10_std', '20170602', '20170603')

    # for record in records:
    #     logger.info('{}'.format(record))

    # logger.info('{}'.format(loggernet.list_tables('__statistics__')))
    # logger.info('{}'.format(loggernet.list_files('draker_')))

    # table_list = loggernet.list_tables('draker')
    table_list = loggernet.list_tables('p3088_01-chikunishi_mega_solar_nihondensetu-chikusei')
    # logger.info('{}'.format(table_list))
    # try:
    #     table_list.remove('Public')
    # except ValueError:
    #     raise
    # finally:
    #     logger.info('{}'.format(table_list))

    # # test for getValue
    # logger.info('{}'.format(loggernet.getValue(station_name, '.Status.DataFillDays(' + str(1) + ')')))



import pytest
from logging import getLogger, DEBUG, INFO

from corautil.utils import extract_data, remove_quotes


@pytest.mark.parametrize(('command', 'cora_output', 'response'),
                         [
                             (
                                    'get-device-setting',
                                    '*get-device-setting,active\n'
                                    '{\n'
                                    '0 {19900101 00:00:00.000} 300000 120000 3 86400000\n'
                                    '}\n'
                                    '+get-device-setting\n',
                                    ['0 {19900101 00:00:00.000} 300000 120000 3 86400000']
                              ),
                             (
                                    'list-stations',
                                    '*list-stations\n'
                                    '{\n'
                                    '{{s0000_draker_engineering_das-4} 3}\n'
                                    '{{LAB-CR200X} 8}\n'
                                    '{{LAB-CR6} 10}\n'
                                    '{{draker_dealer-dot-com} 12}\n'
                                    '{{s0000_draker_engineering_das-1} 14}\n'
                                    '{{Apps_Test_CR1000} 18}\n'
                                    '{{s0000_draker_engineering_das-3} 20}\n'
                                    '{{s0000_draker_engineering_das-2} 22}\n'
                                    '{{s0000_draker_engineering_das-5} 28}\n'
                                    '{{PV2001_CR1000} 30}\n'
                                    '{{CR200Series} 32}\n'
                                    '{{CR300Series} 34}\n'
                                    '{{s0000_draker_engineering_das-6} 36}\n'
                                    '{{__Statistics__} 37}\n'
                                    '}\n'
                                    '+list-stations\n',
                                    [
                                        '{{s0000_draker_engineering_das-4} 3}',
                                        '{{LAB-CR200X} 8}',
                                        '{{LAB-CR6} 10}',
                                        '{{draker_dealer-dot-com} 12}',
                                        '{{s0000_draker_engineering_das-1} 14}',
                                        '{{Apps_Test_CR1000} 18}',
                                        '{{s0000_draker_engineering_das-3} 20}',
                                        '{{s0000_draker_engineering_das-2} 22}',
                                        '{{s0000_draker_engineering_das-5} 28}',
                                        '{{PV2001_CR1000} 30}',
                                        '{{CR200Series} 32}',
                                        '{{CR300Series} 34}',
                                        '{{s0000_draker_engineering_das-6} 36}',
                                        '{{__Statistics__} 37}'
                                    ]
                             ),
                             (
                                 'clock-check',
                                 '*clock-check\n'
                                 '{\n'
                                 '"2017-06-13 16:32:17.330",-2332\n'
                                 '}\n'
                                 '+clock-check,clock checked\n',
                                 [
                                    '"2017-06-13 16:32:17.330",-2332'
                                 ]
                             ),
                             (
                                 'get-device-setting',
                                 'CoraScript 1, 17, 02\n'
                                 '+connect,"coralib3.dll version 1, 11, 05"\n'
                                 '+lock-network\n'
                                 '*get-device-setting,active\n'
                                 '{\n'
                                 '\n'
                                 '}\n'
                                 '+get-device-setting\n'
                                 '+unlock-network\n',
                                 ['']
                             ),
                             (
                                'list-tables',
                                'CoraScript 1, 19, 01\n'
                                '+connect,"coralib3.dll version 1, 11, 05"\n'
                                '+lock-network\n'
                                '*list-tables,p3088_01-chikunishi_mega_solar_nihondensetu-chikusei\n'
                                '{\n'
                                '"fifteenMin"\n'
                                '"panelStatus"\n'
                                '"Public"\n'
                                '"Status"\n'
                                '}\n'
                                '+list-tables\n'
                                '+unlock-network\n',
                                [
                                    '"fifteenMin"',
                                    '"panelStatus"',
                                    '"Public"',
                                    '"Status"'
                                ]
                             )
                         ]
                         )
def test_extract_data(command, cora_output, response):
    logger = getLogger('test_device_settings')
    logger.setLevel(DEBUG)

    extracted_response = extract_data(cora_output, command)
    logger.debug('{}'.format(extracted_response))

    assert extracted_response == response


# @pytest.mark.parametrize(('cora_output', 'string'),
#                          [
#                              'CoraScript 1, 19, 01\n'
#                              '+connect,"coralib3.dll version 1, 11, 05"\n'
#                              '+lock-network\n'
#                              '*get-value\n'
#                              '{\n'
#                              '4,194,304\n'
#                              '}\n'
#                              '+get-value\n'
#                              '+unlock-network\n'
#                          ]
#                          )
# def test_remove_quotes():
#     logger = getLogger('test_device_settings')
#     logger.setLevel(DEBUG)
